	processor 6502
	include macro.h
	include vcs.h

PAL		equ	1
SKIPINTRO	equ	0	;if 1, don't play intro
SECRET	equ	0		;if 1, hide everything!

#if PAL

;PAL
VBLNK	equ	48
LINES	equ	228
OVERSCN	equ	36
FPS		equ	50

#else

;NTSC
VBLNK	equ	40
LINES	equ	192
OVERSCN	equ	30
FPS		equ	60

#endif

FRAMES	equ	6				;total number of frames of animation
STARFRAMES	equ	6			;number of star frames
KSIZE	equ	2				;1 or 2, but 1 no longer works properly
DO_2ND_LINE_WSYNC	equ	0	;if 1, then do WSYNC instead of SLEEPing
							;to the next line when KSIZE == 2
							;this is cheaper on ROM
PAWSH	equ	4
FONTH	equ	8

STARH	equ	7				;height of star graphics
STARK	equ	5-KSIZE			;star kernel size
STARL	equ	(STARK*STARH+3)	;number of lines use by each star
STARTOP	equ	2+PAL			;stars on top
STARBOT	equ	2				;stars on bottom
NSTARS	equ	STARTOP+STARBOT	;number of stars above and below the cat
STARSPD	equ	6				;number of pixels the stars move each frame
STARXM	equ	150				;max star X position

LINES2	equ	LINES - NSTARS*STARL - (3+PAWSH)*KSIZE - 6 - 4 - 2*FONTH - 4

VBLNK64	equ	VBLNK*19/16		;value to set TIM64 to for VBLNK
OVRSC64	equ	OVERSCN*19/16	;ditto for overscan (only used for NTSC)
KLINES	equ LINES2/KSIZE
TAILH	equ	19
TOP		equ (KLINES-TAILH)/2
BOTTOM	equ KLINES - TOP - TAILH

HEADH	equ	14
TARTW	equ	2				;2 or 1, but 1 no longer works properly
TAILGH	equ	8				;height of tail graphics

STARNSZ	equ	$05				;NUSIZx for stars
HEADNSZ	equ	$07				;NUSIZx for head

TEXTCLR	equ	$0E

;colors
#if PAL
DBLUE	equ $B2
TEMPO	equ	195				;fine tempo adjustment
#else
DBLUE	equ $92
TEMPO	equ	162				;fine tempo adjustment
#endif

#if SECRET
RED		equ	DBLUE
ORANGE	equ	DBLUE
YELLOW	equ	DBLUE
GREEN	equ	DBLUE
BLUE	equ	DBLUE
VIOLET	equ	DBLUE
PINK	equ	DBLUE
STARCLR	equ	DBLUE
HEADCLR	equ	DBLUE
HEADBK	equ	DBLUE
TARTBRD	equ	DBLUE
#else

STARCLR	equ	$0E
HEADCLR	equ	$08
HEADBK	equ	$00
TARTBRD	equ	$00

#if PAL
;six rainbox colors (no indigo)
RED		equ $64
ORANGE	equ $48
YELLOW	equ $3A
GREEN	equ $74
BLUE	equ $D6
VIOLET	equ	$C4

PINK	equ	$88
#else
RED		equ $44
ORANGE	equ $28
YELLOW	equ $1A
GREEN	equ $C4
BLUE	equ $86
VIOLET	equ	$64

PINK	equ	$58
#endif
#endif

	SEG.U VARS
	ORG $80

;RAM

frame	ds	1
animfrm	ds	1				;frame of animation (0-5)
rainpar	ds	1				;rainbow parity
rborder	ds	1				;graphics for right border
gfx0	ds	TAILH
gfx1	ds	TAILH
headx	ds	1
enamptr	ds	2				;pointer to table with ENAM0 values for tail
starx	ds	NSTARS			;star X positions
starend	ds	1
tartup	ds	1				;1 -> tart is in upper position
pawsh	ds	1
pawsfrm	ds	1

temp	ds	3

	;song player vars
temp16L equ temp + 1
temp16H equ temp + 2
note1	ds	1
note2	ds	1
vol1	ds	1
vol2	ds	1
sound1	ds	1
sound2	ds	1
beat	ds	1
tempoCount	ds	1
measure	ds	1
atten	ds	1
tempo2	ds	1				;for finely adjusting tempo

textptr	ds	2
textx	ds	1				;0..7, to handle scrolling text
text0	ds	FONTH
text1	ds	FONTH
text2	ds	FONTH
text3	ds	FONTH
text4	ds	FONTH
text5	ds	FONTH
dotext	ds	1				;don't do text until we've done the song once

cherptr	ds	2				;pointer to HMBL/ENABL table, for cherries

	echo "RAM:", ($100 - *), "bytes left"

	SEG CODE
	org $F000
	include tables.asm

	org $F100

	;just pretend this macro sits pasted at line 423 below
	;I had to move this code up here for kernel.asm to be able to make use of it via macro magic
	MAC NTSC_OVERSCAN_PART
	inc frame

	;advance animation frame every so often
	lda frame
	and #$03
	bne DoneSettingAnimationFrame

	;flip rainbox parity every two animation frames
	lda animfrm
	and #1
	beq NoParityAdjust
	eor rainpar
	sta rainpar

NoParityAdjust
	inc animfrm
	lda animfrm
	cmp #FRAMES
	bne DoneSettingAnimationFrame
	lda #0
	sta animfrm

DoneSettingAnimationFrame
	ldx animfrm
	lda TartUpTab,X
	sta tartup

	lda #PAWSH-1
	clc
	adc tartup
	sta pawsh

	;left border
	lda #52
	ldx #3		;M1
	jsr PositionObject

	lda #60
	ldx #4		;BL
	jsr PositionObject
	lda #<(CherryTab-1)
	sec
	sbc tartup
	sta cherptr
	lda #>(CherryTab-1)
	sta cherptr+1

	;head setup
	ldx animfrm
	lda HeadXTab,X
	tay

#if TARTW == 2
	asl
#endif
	asl
	clc
	adc #66+(TARTW-1)*2-9
	sta headx
	adc #9
	ldx #0
	jsr PositionObject

	tya
	bne HeadIsShifted
	ldy #%00001000
	.byte $0C
HeadIsShifted
	ldy #%00010000
	sty rborder

	ldx #TAILH
ClearHeadGFXLoop
	lda #0
	sta gfx0-1,X
	sty gfx1-1,X
	dex
	bne ClearHeadGFXLoop

	ldx animfrm
	lda HeadYTab,X
	asl
	adc #15
	tax
	ldy #HEADH
HeadGFXLoop
	lda rborder
	ora HeadBack-1,Y
	sta gfx1-1,X
	lda Head-1,Y
	sta gfx0-1,X
	dex
	dey
	bne HeadGFXLoop

	ldx animfrm
	lda PawsFrmTab,X
	asl
	asl
	sta pawsfrm

	lda TailNUSIZ,X
	sta NUSIZ0

	lda TailX,X
	ldx #2		;M0
	jsr PositionObject

	sta WSYNC
	sta HMOVE

	;enamptr = TailENAM0Tab - 1 + animfrm*TAILH
	lda #<(TailENAM0Tab-1)
	sta enamptr
	lda #>(TailENAM0Tab-1)
	sta enamptr+1

	ldx animfrm
	beq EnamPtrDone
	clc
EnamPtr_Loop
	lda enamptr
	adc #TAILH
	sta enamptr
	bcc EnamPtr_Next
	inc enamptr+1
EnamPtr_Next
	dex
	bne EnamPtr_Loop

EnamPtrDone
	lda #STARCLR
	sta COLUP1

	lda #STARNSZ
	sta NUSIZ1

#if TARTW == 1
	lda #$F0
	sta PF2
#endif

	sta HMCLR
	ENDM

	include kernel.asm

	echo "Kernel:",*-$F100,"bytes"

	;the kernel grew for the NTSC port, hence it needs another page
	;this is because we do part of the logic during overscan
#if PAL
	echo "Page $F3-$F4:",$F500-*,"bytes left"
	org $F500
#else
	echo "Page $F3-$F5:",$F600-*,"bytes left"
	org $F600
#endif
	.byte 0				;align

	MAC TAIL_ENAM_TAB
.Start	SET	{1}
.End	SET	{2}
.Pos	SET  0
	REPEAT TAILH
		IF .Start <= .Pos && .End > .Pos
			.byte 2
		ELSE
			.byte 0
		ENDIF
.Pos	SET	.Pos + 1
	REPEND
	ENDM

TailENAM0Tab
	TAIL_ENAM_TAB 6, 9
	TAIL_ENAM_TAB 6, 7
	TAIL_ENAM_TAB 4, 6
	TAIL_ENAM_TAB 3, 6
	TAIL_ENAM_TAB 5, 6
	TAIL_ENAM_TAB 6, 8

CherryTab
	.byte 0
	.byte $42
	.byte $C2
	.byte $32
	.byte $22
	.byte $32
	.byte $A2
	.byte $22
	.byte $D2
	.byte $52
	.byte $92
	.byte $32
	.byte $52
	.byte $B2
	.byte $72
	.byte $D2
	.byte $82
	.byte $32
	.byte 0

	include font.h
	include numberfont.h

TartUpTab
	.byte 1
	.byte 1
	.byte 0
	.byte 0
	.byte 0
	.byte 0

	include song.asm
	include songplay.h
	include text.asm

Start
	CLEAN_START

#if SKIPINTRO
	lda #INTROLENGTH
	sta measure
#endif

	;init stars
	ldx #NSTARS
InitStars
	adc starx,X
	adc starx+1,X
	adc #50
	and #63
	sta starx-1,X
	dex
	bne InitStars

	lda #1
	sta VDELP0

	lda #<Text
	sta textptr
	lda #>Text
	sta textptr+1

MainLoop
	VERTICAL_SYNC

	lda #VBLNK64
	sta TIM64T	

#if PAL
	NTSC_OVERSCAN_PART
#endif

	lda tempo2
	clc
	adc #TEMPO
	sta tempo2
	bcs PlayingSong
	jmp NotPlayingSong

PlayingSong
	;play song
	jmp songPlayer
SongPlayerRet
	lda dotext
	bne NotLastMeasure

	;start doing the text scroller at measure STARTDOTEXT
	lda measure
	cmp #STARTDOTEXT
	bne NotLastMeasure
	lda #1
	sta dotext

NotLastMeasure
	lda dotext
	cmp #1
	beq DoText
	jmp DoneText

DoText
	;shift text left
	ldx #FONTH-1
ShiftText
	asl text5,X
	rol text4,X
	rol text3,X
	rol text2,X
	rol text1,X
	rol text0,X
	dex
	bpl ShiftText

	;advance text pointer if we should
	lda textx
	cmp #8
	bne TextPtrOK

	lda #0
	sta textx

	inc textptr
	bne TextPtrInced
	inc textptr+1

TextPtrInced
	;restart if we reached TextEnd
	lda textptr
	cmp #<TextEnd
	bne TextPtrOK
	lda textptr+1
	cmp #>TextEnd
	bne TextPtrOK

	lda #2
	sta dotext
	jmp DoneText

TextPtrOK
	;load current text column
	ldy #0
	lda (textptr),Y

	;set up temp to point at the current character
	cmp #64
	bcs NormalCharacter
	cmp #44
	bcs Number
	cmp #41
	beq Parens

	sec
	sbc #32
	asl
	asl
	asl
	adc #<NumberBlank
	sta temp
	lda #0
	adc #>NumberBlank
	sta temp+1
	jmp DoMaskInFont

Parens
	lda #<CharCloseParens
	sta temp
	lda #>CharCloseParens
	sta temp+1
	jmp DoMaskInFont

Number
	;number
	sec
	sbc #44				;','
	asl
	asl
	asl
	clc
	adc #<CharComma
	sta temp
	lda #0
	adc #>CharComma
	sta temp+1
	jmp DoMaskInFont

NormalCharacter
	sec
	sbc #65				;'A'
	asl
	asl
	asl
	clc
	adc #<Font
	sta temp
	lda #0
	adc #>Font
	sta temp+1

DoMaskInFont
	;textx is the bit to pick, in left-to-right order
	ldy #FONTH-1
MaskInFont
	lda (temp),Y
	ldx textx
	inx
MaskInFont_Shift
	asl
	dex
	bne MaskInFont_Shift

	;graphics bit is in carry
	lda #0
	rol
	ora text5,Y
	sta text5,Y

	dey
	bpl MaskInFont

	inc textx
DoneText

NotPlayingSong
	;handle stars
	ldx #NSTARS
StarsLoop
	;move left
	txa
	and #1
	clc
	adc starx-1,X
	sec
	sbc #STARSPD
	bcs StarsLoop_StoreX
	lda #STARXM
StarsLoop_StoreX
	sta starx-1,X
	dex
	bne StarsLoop

	;reset stuff ruined by TextKernel
	lda #HEADCLR
	sta COLUP0
	lda #0
	sta VDELP1

WaitForVblankEnd
#if !PAL
	;This WSYNC fixes the shaky image in NTSC mode
	sta WSYNC
#endif
	lda INTIM
	bne WaitForVblankEnd	

	sta WSYNC
	sta HMOVE

#if !PAL
	;We're not burning enough lines in NTSC mode for some reason
	sta WSYNC
	sta HMOVE
#endif

	sta VBLANK
	jmp AlignedKernel

StarKernel
StarKernel_NextStar
	sta WSYNC
	sta HMOVE
	
	;set up pointer to frame in temp
	;frame number = (animfrm + X) % STARFRAMES
	txa
	clc
	adc animfrm
	cmp #STARFRAMES
	bcc StarKernel_FrameOK
	sbc #STARFRAMES				;not BCC -> carry is set
StarKernel_FrameOK
	sta temp+2
	asl
	asl
	asl
	sec
	sbc temp+2
	clc
	adc #<(Star0-1)
	sta temp
	lda #0
	adc #>(Star0-1)
	sta temp+1

	lda starx,X
	stx temp+2
	ldx #1
	jsr PositionObject2

	ldx temp+2
	SLEEP 20
	sta HMCLR

	ldy #STARH
StarKernel_Loop
	sta WSYNC
	sta HMOVE
	lda (temp),Y
	sta GRP1

	sta WSYNC
	sta HMOVE
	sta WSYNC
	sta HMOVE
#if KSIZE == 1
	sta WSYNC
	sta HMOVE
#endif

	dey
	bne StarKernel_Loop

	lda #0
	sta GRP1

	inx
	cpx starend
	bne StarKernel_NextStar

	cpx #STARTOP
	beq StarKernel_GotoTop
	jmp StarKernelRetBot
StarKernel_GotoTop
	jmp StarKernelRetTop
	echo "StarKernel:",StarKernel,"-",*

	align 32
	;Positions object X to horizontal position in accumulator
	; 0 = P0
	; 1 = P1
	; 2 = M0
	; 3 = M1
	; 4 = BL
PositionObject
	clc
	sta WSYNC
DivideLoop
	sbc #15
	bcs DivideLoop

	;The might become unaligned without us noticing, causing odd
	;mistimings in the display code.
	;Print the page difference so this can be more easily spotted.
	echo "DivideLoop page difference:", (>DivideLoop - >*)

	eor #7
	asl
	asl
	asl
	asl
	sta HMP0,X
	sta RESP0,X
	rts
	echo "PositionObject:",PositionObject,"-",*

PositionObject2
	clc
	sta WSYNC
	sta HMOVE
DivideLoop2
	sbc #15
	bcs DivideLoop2

	;The might become unaligned without us noticing, causing odd
	;mistimings in the display code.
	;Print the page difference so this can be more easily spotted.
	echo "DivideLoop2 page difference:", (>DivideLoop - >*)

	eor #7
	asl
	asl
	asl
	asl
	sta HMP0,X
	sta RESP0,X
	sta WSYNC
	sta HMOVE
	rts
	echo "PositionObject2:",PositionObject2,"-",*

	echo "ROM:", ($FFFC - *), "bytes left"

	org $FFFC
	.word Start
	.word Start
